@@ -41,11 +41,14 @@
 #include "font-model.h"
 #include "sushi-font-widget.h"
 
-#define FONT_VIEW_TYPE_APPLICATION font_view_application_get_type()
-#define FONT_VIEW_APPLICATION(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST ((obj), FONT_VIEW_TYPE_APPLICATION, FontViewApplication))
+#define FONT_VIEW_TYPE_APPLICATION (font_view_application_get_type ())
+#define FONT_VIEW_ICON_NAME "org.gnome.font-viewer"
 
-typedef struct {
+G_DECLARE_FINAL_TYPE (FontViewApplication, font_view_application,
+                      FONT_VIEW, APPLICATION,
+                      GtkApplication)
+
+struct _FontViewApplication {
     GtkApplication parent;
 
     GtkWidget *main_window;
@@ -56,6 +59,10 @@ typedef struct {
     GtkWidget *font_widget;
     GtkWidget *info_button;
     GtkWidget *install_button;
+    GtkWidget *install_label;
+    GtkWidget *installing_label;
+    GtkWidget *installed_label;
+    GtkWidget *install_failed_label;
     GtkWidget *back_button;
     GtkWidget *stack;
     GtkWidget *swin_view;
@@ -65,16 +72,15 @@ typedef struct {
     GtkWidget *search_bar;
     GtkWidget *search_entry;
     GtkWidget *search_toggle;
+    GtkWidget *menu_button;
 
     GtkTreeModel *model;
     GtkTreeModel *filter_model;
 
     GFile *font_file;
-} FontViewApplication;
 
-typedef struct {
-    GtkApplicationClass parent_class;
-} FontViewApplicationClass;
+    GCancellable *cancellable;
+};
 
 static gboolean
 _print_version_and_exit (const gchar *option_name,
@@ -94,7 +100,8 @@ static const GOptionEntry goption_options[] =
     { NULL }
 };
 
-G_DEFINE_TYPE (FontViewApplication, font_view_application, GTK_TYPE_APPLICATION);
+G_DEFINE_TYPE (FontViewApplication, font_view_application,
+               GTK_TYPE_APPLICATION)
 
 static void font_view_application_do_overview (FontViewApplication *self);
 static void ensure_window (FontViewApplication *self);
@@ -511,22 +518,29 @@ install_button_refresh_appearance (FontViewApplication *self,
 {
     FT_Face face;
     GtkStyleContext *context;
+    GtkWidget *current_label;
 
     context = gtk_widget_get_style_context (self->install_button);
 
+    current_label = gtk_bin_get_child (GTK_BIN (self->install_button));
+    gtk_container_remove (GTK_CONTAINER (self->install_button), current_label);
+
     if (error != NULL) {
-        gtk_button_set_label (GTK_BUTTON (self->install_button), _("Install Failed"));
+        gtk_container_add (GTK_CONTAINER (self->install_button), self->install_failed_label);
         gtk_widget_set_sensitive (self->install_button, FALSE);
         gtk_style_context_remove_class (context, "suggested-action");
     } else {
         face = sushi_font_widget_get_ft_face (SUSHI_FONT_WIDGET (self->font_widget));
 
         if (font_view_model_get_iter_for_face (FONT_VIEW_MODEL (self->model), face, NULL)) {
-            gtk_button_set_label (GTK_BUTTON (self->install_button), _("Installed"));
+            gtk_container_add (GTK_CONTAINER (self->install_button), self->installed_label);
             gtk_widget_set_sensitive (self->install_button, FALSE);
             gtk_style_context_remove_class (context, "suggested-action");
+        } else if (self->cancellable != NULL) {
+            gtk_container_add (GTK_CONTAINER (self->install_button), self->installing_label);
+            gtk_widget_set_sensitive (self->install_button, FALSE);
         } else {
-            gtk_button_set_label (GTK_BUTTON (self->install_button), _("Install"));
+            gtk_container_add (GTK_CONTAINER (self->install_button), self->install_label);
             gtk_widget_set_sensitive (self->install_button, TRUE);
             gtk_style_context_add_class (context, "suggested-action");
         }
@@ -549,6 +563,8 @@ font_install_finished_cb (GObject      *source_object,
         g_debug ("Install failed: %s", err->message);
         g_error_free (err);
     }
+
+    g_clear_object (&self->cancellable);
 }
 
 static void
@@ -634,9 +650,20 @@ install_button_clicked_cb (GtkButton *button,
     dest_file = g_file_get_child (dest_location, dest_filename);
     g_free (dest_filename);
 
+    self->cancellable = g_cancellable_new ();
+
     /* TODO: show error dialog if file exists */
-    g_file_copy_async (self->font_file, dest_file, G_FILE_COPY_NONE, 0, NULL, NULL, NULL,
-                       font_install_finished_cb, self);
+    g_file_copy_async (self->font_file,
+                       dest_file,
+                       G_FILE_COPY_NONE,
+                       G_PRIORITY_DEFAULT,
+                       self->cancellable,
+                       NULL,
+                       NULL,
+                       font_install_finished_cb,
+                       self);
+
+    install_button_refresh_appearance (self, NULL);
 
     g_object_unref (dest_file);
     g_object_unref (dest_location);
@@ -798,6 +825,7 @@ font_view_application_do_open (FontViewApplication *self,
                                GFile *file,
                                gint face_index)
 {
+    GtkSizeGroup *install_size_group;
     GtkWidget *back_image;
     gchar *uri;
 
@@ -805,7 +833,23 @@ font_view_application_do_open (FontViewApplication *self,
 
     /* add install button */
     if (self->install_button == NULL) {
-        self->install_button = gtk_button_new_with_label (_("Install"));
+        install_size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+
+        self->install_label = g_object_ref_sink (gtk_label_new (_("Install")));
+        gtk_widget_show (self->install_label);
+        gtk_size_group_add_widget (install_size_group, self->install_label);
+        self->installing_label = g_object_ref_sink (gtk_label_new (_("Installing…")));
+        gtk_widget_show (self->installing_label);
+        gtk_size_group_add_widget (install_size_group, self->installing_label);
+        self->installed_label = g_object_ref_sink (gtk_label_new (_("Installed")));
+        gtk_widget_show (self->installed_label);
+        gtk_size_group_add_widget (install_size_group, self->installed_label);
+        self->install_failed_label = g_object_ref_sink (gtk_label_new (_("Failed")));
+        gtk_widget_show (self->install_failed_label);
+        gtk_size_group_add_widget (install_size_group, self->install_failed_label);
+
+        self->install_button = gtk_button_new ();
+        gtk_container_add (GTK_CONTAINER (self->install_button), self->install_label);
         gtk_widget_set_valign (self->install_button, GTK_ALIGN_CENTER);
         gtk_style_context_add_class (gtk_widget_get_style_context (self->install_button),
                                      "text-button");
@@ -813,6 +857,8 @@ font_view_application_do_open (FontViewApplication *self,
 
         g_signal_connect (self->install_button, "clicked",
                           G_CALLBACK (install_button_clicked_cb), self);
+
+        g_object_unref (install_size_group);
     }
 
     if (self->info_button == NULL) {
@@ -842,6 +888,7 @@ font_view_application_do_open (FontViewApplication *self,
     }
 
     gtk_widget_hide (self->search_toggle);
+    gtk_widget_hide (self->menu_button);
 
     uri = g_file_get_uri (file);
 
@@ -928,11 +975,17 @@ font_view_application_do_overview (FontViewApplication *self)
     }
 
     if (self->install_button) {
+        g_clear_pointer (&self->install_label, gtk_widget_destroy);
+        g_clear_pointer (&self->installing_label, gtk_widget_destroy);
+        g_clear_pointer (&self->installed_label, gtk_widget_destroy);
+        g_clear_pointer (&self->install_failed_label, gtk_widget_destroy);
+
         gtk_widget_destroy (self->install_button);
         self->install_button = NULL;
     }
 
     gtk_widget_show (self->search_toggle);
+    gtk_widget_show (self->menu_button);
 
     font_view_ensure_model (self);
 
@@ -1064,7 +1117,7 @@ action_about (GSimpleAction *action,
                            "authors", authors,
                            "program-name", _("Fonts"),
                            "comments", _("View fonts on your system"),
-                           "logo-icon-name", "preferences-desktop-font",
+                           "logo-icon-name", FONT_VIEW_ICON_NAME,
                            "translator-credits", _("translator-credits"),
                            "license-type", GTK_LICENSE_GPL_2_0,
                            "wrap-license", TRUE,
@@ -1088,6 +1141,8 @@ static void
 ensure_window (FontViewApplication *self)
 {
     GtkWidget *window, *swin, *box, *image;
+    GtkBuilder *builder;
+    GMenuModel *menu;
 
     if (self->main_window)
         return;
@@ -1095,7 +1150,7 @@ ensure_window (FontViewApplication *self)
     self->main_window = window = gtk_application_window_new (GTK_APPLICATION (self));
     gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
     gtk_window_set_default_size (GTK_WINDOW (window), 800, 600);
-    gtk_window_set_icon_name (GTK_WINDOW (window), "preferences-desktop-font");
+    gtk_window_set_icon_name (GTK_WINDOW (window), FONT_VIEW_ICON_NAME);
 
     self->header = gtk_header_bar_new ();
     gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header), TRUE);
@@ -1118,6 +1173,21 @@ ensure_window (FontViewApplication *self)
     box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
     gtk_stack_add_named (GTK_STACK (self->stack), box, "overview");
 
+    builder = gtk_builder_new ();
+    gtk_builder_add_from_resource (builder, "/org/gnome/font-viewer/font-view-app-menu.ui", NULL);
+    menu = G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu"));
+
+    self->menu_button = gtk_menu_button_new ();
+    image = gtk_image_new_from_icon_name ("open-menu-symbolic", GTK_ICON_SIZE_BUTTON);
+    gtk_widget_show (image);
+    gtk_container_add (GTK_CONTAINER (self->menu_button), image);
+    gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (self->menu_button), menu);
+    gtk_widget_set_no_show_all (self->menu_button, TRUE);
+    gtk_widget_show (self->menu_button);
+    gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header), self->menu_button);
+
+    g_object_unref (builder);
+
     self->search_bar = gtk_search_bar_new ();
     gtk_container_add (GTK_CONTAINER (box), self->search_bar);
     self->search_entry = gtk_search_entry_new ();
@@ -1158,8 +1228,6 @@ static void
 font_view_application_startup (GApplication *application)
 {
     FontViewApplication *self = FONT_VIEW_APPLICATION (application);
-    GtkBuilder *builder;
-    GMenuModel *menu;
 
     G_APPLICATION_CLASS (font_view_application_parent_class)->startup (application);
 
@@ -1168,13 +1236,6 @@ font_view_application_startup (GApplication *application)
 
     g_action_map_add_action_entries (G_ACTION_MAP (self), action_entries,
                                      G_N_ELEMENTS (action_entries), self);
-    builder = gtk_builder_new ();
-    gtk_builder_add_from_resource (builder, "/org/gnome/font-viewer/font-view-app-menu.ui", NULL);
-    menu = G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu"));
-    gtk_application_set_app_menu (GTK_APPLICATION (self), menu);
-
-    g_object_unref (builder);
-    g_object_unref (menu);
 }
 
 static void
@@ -1191,6 +1252,9 @@ font_view_application_dispose (GObject *obj)
 {
     FontViewApplication *self = FONT_VIEW_APPLICATION (obj);
 
+    g_cancellable_cancel (self->cancellable);
+
+    g_clear_object (&self->cancellable);
     g_clear_object (&self->font_file);
     g_clear_object (&self->filter_model);
     g_clear_object (&self->model);
